from pybedtools import BedTool
from collections import defaultdict
import os
from os.path import join, exists

GTF_GENE_FEATURE_INDEX = 2  # Position of the "feature" parameter in an interval (2 in GTF format)

gtf = BedTool(snakemake.input.gtf)

biotype_dict = defaultdict(list)
biotype_gene_dict = defaultdict(list)
results_log_dict = defaultdict(list)

# Run through all the intervals, adding to the pertinent dictionaries the intervals from each biotype
for interval in gtf:
    # This is necessary because the following line changes the order of attributes,
    # and this can lead to segtools crashing
    fields = interval.fields
    biotype = interval.attrs['gene_biotype']  # gene_type instead of gene_biotype in GH19
    biotype_dict[biotype].append(fields)
    if interval[GTF_GENE_FEATURE_INDEX] == 'gene':
        biotype_gene_dict[biotype].append(fields)

# Create all the files with all the intervals from each biotype, and add their sizes to the results log dictionary
for biotype, array in biotype_dict.items():
    if not exists(join(snakemake.params.outdir, "general", biotype)):
        os.makedirs(join(snakemake.params.outdir, "general", biotype))
    BedTool(array).moveto(join(snakemake.params.outdir, "general", biotype, snakemake.params.outfile))
    results_log_dict[biotype].append(len(array))

# Create the files with only the intervals that are genes from each biotype
# and add their sizes to the results log dictionary
for biotype, array in biotype_gene_dict.items():
    if not exists(join(snakemake.params.outdir, "gene", biotype)):
        os.makedirs(join(snakemake.params.outdir, "gene", biotype))
    BedTool(array).moveto(join(snakemake.params.outdir, "gene", biotype, snakemake.params.outfile))
    results_log_dict[biotype].append(len(array))

# Write down the resulting files' sizes in a new file in the "logs" folder
log_file = open(snakemake.log[0], 'w')
log_file.write("gene biotype\tannotations\tgenes\n")
for biotype, values in results_log_dict.items():
    log_file.write(biotype + "\t" + '{}'.format(values[0]) + "\t" + '{}'.format(values[1]) + "\n")
log_file.close()



import argparse
import sys
import snakemake

from .version import __version__
from os import path


# TODO add build and species to the path of the output so that different runs in the same folder can be done
def main(args=sys.argv[1:]):
    here = path.abspath(path.dirname(__file__))
    default_prefix = path.dirname(path.dirname(path.dirname(path.dirname(here))))

    parser = argparse.ArgumentParser(description='Download necessary files, run workflow and obtain results',
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('--version', action='version', version=__version__)
    parser.add_argument('segmentation', help='.bed.gz file, the segmentation/annotation output from Segway')
    parser.add_argument('--parameters', default=False,
                        help='The params.params file used to obtain the gmtk-parameters')
    parser.add_argument('-o', '--outdir', default='outdir', help='Output directory to store all the results')
    parser.add_argument('--species', default='Homo_sapiens', help='Species of the genome used for the segmentation')
    parser.add_argument('--build', default='hg38', help='Build of the genome assembly used for the segmentation')
    parser.add_argument('--prefix', default=default_prefix,
                        help='Prefix where all the GGD data is going to be downloaded')

    parsed_args = parser.parse_args(args)

    snakemake.snakemake(path.join(here, "Snakefile"), config=vars(parsed_args))  # dryrun=True, printshellcmds=True)



import pandas as pd

# creates a data frame with bedtools' output, adds up the parameters for all segments of the same label,
# and writes it in a tab-delimited file
# two rows are added to the table: GC_content and AT_content

usecols = ["4_usercol", "12_num_A", "13_num_C", "14_num_G", "15_num_T", "18_seq_len"]

df = pd.read_table(snakemake.input.infile, header=0, usecols=usecols)
df.columns = ["label", "num_A", "num_C", "num_G", "num_T", "length"]
sums = df.groupby("label").sum()
sums["GC_content"] = sums["num_G"].add(sums["num_C"]).div(sums["length"])
sums["AT_content"] = sums["num_A"].add(sums["num_T"]).div(sums["length"])
sums.to_csv(path_or_buf=snakemake.output.outfile, sep='\t')



import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
from os import path

sns.set()

cmap = sns.diverging_palette(220, 10, as_cmap=True)


# Prepare nucleotide results
res_nuc = pd.read_table(snakemake.input.nuc, index_col=0)['GC_content']
res_nuc.index = res_nuc.index.map(str)

# TODO change directly from matrix:
# 2 nums after decimal point? Range for heatmap between 35% and 65%

# Prepare length_distribution results
# Preparing the annotation for the matrix, creating a new column called 'frac.segs'
res_len = pd.read_table(snakemake.input.len_dist, index_col=0)
res_len['frac.segs'] = (res_len['num.segs'] / res_len.loc['all']['num.segs'])
res_len = res_len.drop(['num.segs', 'num.bp'], axis=1)
res_len = res_len.drop('all')

# Joining both matrix to create final heatmap and annotation
res_ann = res_len.join(res_nuc)
res = res_ann.copy()

# Interpolation of the parameters to rescale them between 0 and 1
# ['mean.len', 'median.len','stdev.len', 'frac.segs', 'frac.bp', 'GC_content']
for col in res.columns:
    res[col] = (res[col] - res[col].min()) / (res[col].max()-res[col].min())

# Prepare the aggregation results for 'protein_coding' and 'lincRNA' and join them
pc_file = next(x for x in snakemake.input.aggs if path.basename(path.dirname(x)) == 'protein_coding')
lrna_file = next(x for x in snakemake.input.aggs if path.basename(path.dirname(x)) == 'lincRNA')

res_agg_pc = pd.read_table(pc_file, index_col=0)
res_agg_linc = pd.read_table(lrna_file, index_col=0)
res_agg_ann = res_agg_pc.join(res_agg_linc, lsuffix='_protein_coding', rsuffix='_lincRNA')
res_agg = res_agg_ann.copy()

for col in res_agg.columns:
    res_agg[col] = (res_agg[col] - res_agg[col].min()) / (res_agg[col].max()-res_agg[col].min())

# Create grid and plot the results in two separate plots
f, (ax1, ax2, ax3) =\
    plt.subplots(1, 3, figsize=(25, 5), sharey=True, gridspec_kw={"wspace": 0.05, "width_ratios": [0.5, 1, 2]})

# Prepare the gmtk parameters
if snakemake.config['parameters']:
    res_gmtk = pd.read_csv(snakemake.input.gmtk, index_col=0)
    sns.heatmap(res_gmtk, cbar=False, ax=ax1)
else:
    f.delaxes(ax1)

# TODO Change formats to strings and format the values in the matrix directly
sns.heatmap(res, annot=res_ann, cbar=False, vmax=1, vmin=0, cmap="YlGnBu", ax=ax2, fmt='.4g') 
sns.heatmap(res_agg, annot=res_agg_ann, cbar=False, cmap=cmap, ax=ax3, fmt='.5g')

ax1.set_title('GMTK_parameters')
ax2.set_title('Length Distribution + GC_Content')
ax2.set_ylabel('')
ax3.set_title('Aggregation (protein_coding + lincRNA)')

plt.gcf().subplots_adjust(bottom=0.5)
# plt.tight_layout()

f.savefig(snakemake.output.outfile)
